@@ -3,12 +3,14 @@ from __future__ import print_function
 
 import clu.abstract
 import copy
-import sys
+import sys, re
 
-from clu.config.abc import FrozenKeyMap, KeyMap, NamespaceWalker
-from clu.config.ns import concatenate_ns, unpack_ns, pack_ns
+from clu.config import abc, ns
+from clu.config.keymaputils import thaw_name, thaw_class, freeze_name, freeze_class
+from clu.constants import consts
+from clu.predicates import tuplize, typeof
 from clu.typology import ismapping
-from clu.exporting import Exporter
+from clu.exporting import Exporter, sysmods, itermodule
 
 exporter = Exporter(path=__file__)
 export = exporter.decorator()
@@ -23,8 +25,8 @@ def flatwalk(mapping):
         Based on https://stackoverflow.com/a/12507546/298171
     """
     for nskey in mapping.keys():
-        key, namespaces = unpack_ns(nskey)
-        yield namespaces + [key, mapping[nskey]]
+        key, fragments = ns.unpack_ns(nskey)
+        yield fragments + [key, mapping[nskey]]
 
 @export
 def articulate(mapping, walker=flatwalk):
@@ -33,20 +35,20 @@ def articulate(mapping, walker=flatwalk):
         walking function.
     """
     tree = {}
-    for *namespaces, key, value in walker(mapping):
+    for *fragments, key, value in walker(mapping):
         d = tree
-        for namespace in namespaces:
+        for fragment in fragments:
             try:
-                d = d[namespace]
+                d = d[fragment]
             except KeyError:
-                d[namespace] = {}
-                d = d[namespace]
+                d[fragment] = {}
+                d = d[fragment]
         d[key] = value
     return tree
 
 @export
-class FrozenFlat(FrozenKeyMap, clu.abstract.ReprWrapper,
-                               clu.abstract.Cloneable):
+class FrozenFlat(abc.FrozenKeyMap, clu.abstract.ReprWrapper,
+                                   clu.abstract.Cloneable):
     
     """ A concrete immutable – or frozen – KeyMap class with a flat internal topology. """
     
@@ -71,6 +73,9 @@ class FrozenFlat(FrozenKeyMap, clu.abstract.ReprWrapper,
         if updates:
             self.dictionary.update(**updates)
     
+    def thaw(self):
+        return thaw_class(type(self))(dictionary=copy.deepcopy(self.dictionary))
+    
     def nestify(self, cls=None, walker=flatwalk):
         """ Articulate a flattened KeyMap instance out into one that is nested. """
         if cls is None:
@@ -104,12 +109,12 @@ class FrozenFlat(FrozenKeyMap, clu.abstract.ReprWrapper,
         return copy.deepcopy(self.dictionary)
 
 @export
-class Flat(FrozenFlat, KeyMap):
+class Flat(FrozenFlat, abc.KeyMap):
     
     """ A concrete mutable KeyMap class with a flat internal topology. """
     
     def freeze(self):
-        return FrozenFlat(dictionary=copy.deepcopy(self.dictionary))
+        return freeze_class(type(self))(dictionary=copy.deepcopy(self.dictionary))
     
     def __setitem__(self, nskey, value):
         self.dictionary[nskey] = value
@@ -147,8 +152,8 @@ def mapwalk(mapping, pre=None):
 # CONCRETE SUBCLASSES: FrozenNested and Nested
 
 @export
-class FrozenNested(NamespaceWalker, clu.abstract.ReprWrapper,
-                                    clu.abstract.Cloneable):
+class FrozenNested(abc.NamespaceWalker, clu.abstract.ReprWrapper,
+                                        clu.abstract.Cloneable):
     
     """ A concrete immutable – or frozen – KeyMap class with an articulated –
         or, if you will, a nested – internal topology.
@@ -171,33 +176,60 @@ class FrozenNested(NamespaceWalker, clu.abstract.ReprWrapper,
         self.tree = dictify(dict(tree or {}), cls=dict)
         if updates:
             for nskey, value in updates.items():
-                key, namespaces = unpack_ns(nskey)
+                key, fragments = ns.unpack_ns(nskey)
                 d = self.tree
-                for namespace in namespaces:
+                for fragment in fragments:
                     try:
-                        d = d[namespace]
+                        d = d[fragment]
                     except KeyError:
-                        d[namespace] = {}
-                        d = d[namespace]
+                        d[fragment] = {}
+                        d = d[fragment]
                 d[key] = value
     
     def walk(self):
         """ Iteratively walk the nested KeyMap’s tree of dicts. """
         yield from mapwalk(self.tree)
     
-    def submap(self, *namespaces, unprefixed=False):
-        """ Return a flattened dict containing only the namespaced items. """
+    def thaw(self):
+        return thaw_class(type(self))(tree=copy.deepcopy(self.tree))
+    
+    def submap(self, *namespaces, unprefixed=False, cls=False):
+        """ Return a flattened mapping, if possible a mutable one,
+            containing only items with keys matching the specified
+            namespaces (as namespaced:key:paths).
+        """
+        # We’ll need this later:
+        if not cls:
+            cls = freeze_class(type(self))
+        
+        # Short-circuit for returning unprefixed top-level items:
         if unprefixed:
-            return { key : value for key, value in self.tree.items() \
-                                  if not ismapping(value) }
+            return cls({ key : value for key, value in self.tree.items() \
+                                                    if not ismapping(value) })
+        
+        # If it’s not unprefixed, and we have no namespaces,
+        # we cough up a new instance with our data:
         if not namespaces:
-            return dict(self)
-        if not concatenate_ns(*namespaces) in self.namespaces():
-            return {}
-        d = self.tree
+            return cls(self.tree)
+        
+        # Our namespaces, their output data:
+        ours = frozenset(self._get_namespace_foset().things)
+        theirs = {}
+        
+        # Go through the namespaces we were passed, and copy anything
+        # we have that matches those namespaces into a new output dict –
+        # wholesale and sans any namespaced-key prefixes:
         for namespace in namespaces:
-            d = d[namespace]
-        return { pack_ns(key, *namespaces) : value for key, value in d.items() }
+            if namespace not in ours:
+                continue
+            d = self.tree
+            for fragment in ns.split_ns(namespace):
+                d = d[fragment]
+            theirs[namespace] = d
+        
+        # Return a (possibly frozen) instance containing the specified
+        # namespaced data, as a namespaced instance:
+        return cls(theirs)
     
     def inner_repr(self):
         return repr(self.tree)
@@ -211,65 +243,100 @@ class FrozenNested(NamespaceWalker, clu.abstract.ReprWrapper,
         return copy.deepcopy(self.tree)
     
     def __contains__(self, nskey):
-        key, namespaces = unpack_ns(nskey)
-        if not namespaces:
+        key, fragments = ns.unpack_ns(nskey)
+        if not fragments:
             # Test for mappings to prevent false positives:
             return not ismapping(self.tree.get(key, {}))
         d = self.tree
-        for namespace in namespaces:
+        for fragment in fragments:
             try:
-                d = d[namespace]
+                d = d[fragment]
             except KeyError:
                 return False
         # Test for mappings to prevent false positives:
         return not ismapping(d.get(key, {}))
     
     def __getitem__(self, nskey):
-        key, namespaces = unpack_ns(nskey)
-        if not namespaces:
+        key, fragments = ns.unpack_ns(nskey)
+        if not fragments:
             return self.tree[key]
         d = self.tree
-        for namespace in namespaces:
-            d = d[namespace]
+        for fragment in fragments:
+            d = d[fragment]
         return d[key]
 
 @export
-class Nested(FrozenNested, KeyMap):
+class Nested(FrozenNested, abc.KeyMap):
     
     """ A concrete mutable KeyMap class with an articulated (or nested)
         internal topology.
     """
     
     def freeze(self):
-        return FrozenNested(tree=copy.deepcopy(self.tree))
+        return freeze_class(type(self))(tree=copy.deepcopy(self.tree))
     
     def __setitem__(self, nskey, value):
-        key, namespaces = unpack_ns(nskey)
-        if not namespaces:
+        key, fragments = ns.unpack_ns(nskey)
+        if not fragments:
             self.tree[key] = value
         else:
             d = self.tree
-            for namespace in namespaces:
+            for fragment in fragments:
                 try:
-                    d = d[namespace]
+                    d = d[fragment]
                 except KeyError:
-                    d[namespace] = {}
-                    d = d[namespace]
+                    d[fragment] = {}
+                    d = d[fragment]
             d[key] = value
     
     def __delitem__(self, nskey):
         if nskey in self:
-            key, namespaces = unpack_ns(nskey)
-            if not namespaces:
+            key, fragments = ns.unpack_ns(nskey)
+            if not fragments:
                 del self.tree[key]
             else:
                 d = self.tree
-                for namespace in namespaces:
-                    d = d[namespace]
+                for fragment in fragments:
+                    d = d[fragment]
                 del d[key]
         else:
             raise KeyError(nskey)
 
+@export
+class KeyedAccessor(metaclass=clu.abstract.Slotted):
+    
+    __slots__ = ('keymap', 'namespace')
+    
+    def __init__(self, keymap, namespace=""):
+        ourmap = hasattr(keymap, 'nestify') and keymap.nestify() or keymap
+        cls = type(keymap)
+        self.keymap = cls(ourmap.submap(self.namespace, unprefixed=bool(self.namespace)))
+    
+    def __getattr__(self, key):
+        """ Attribute access repacks the key """
+        nskey = ns.pack_ns(key, *self.namespace)
+        if nskey in self.keymap:
+            return self[nskey]
+        raise AttributeError(f"namespaced key {nskey} not present in keymap")
+    
+    def accessor(self, *addenda):
+        namespace = ns.concatenate_ns(*self.namespace, *addenda)
+        realigned = type(self)(keymap=self.keymap,
+                               namespaces=ns.concatenate_ns(*namespace))
+        return realigned
+    
+    def __contains__(self, nskey):
+        return nskey in self.keymap
+    
+    def __getitem__(self, nskey):
+        """ Unpack and return """
+        *addenda, key = ns.split_ns(nskey)
+        accessor = self.accessor(*addenda)
+        if nskey in accessor.keymap:
+            return accessor.keymap[nskey]
+        if ns.concatenate_ns(*addenda) in set(self.keymap.namespaces()):
+            return accessor
+
 # Assign the modules’ `__all__` and `__dir__` using the exporter:
 __all__, __dir__ = exporter.all_and_dir()
 
@@ -305,11 +372,36 @@ def flatdict():
     """ Flat-dictionary fixture function """
     out = {}
     for mappingpath in mapwalk(nestedmaps()):
-        *namespaces, key, value = mappingpath
-        nskey = pack_ns(key, *namespaces)
+        *fragments, key, value = mappingpath
+        nskey = ns.pack_ns(key, *fragments)
         out[nskey] = value
     return out
 
+@inline.fixture
+def frozenclasses():
+    """ Scan `sys.modules` for “frozen”-looking types """
+    from clu.naming import qualified_name
+    seen = set()
+    out = list()
+    for module in sysmods():
+        for key, value in itermodule(module):
+            if key.startswith("rozen", 1):
+                if key not in seen:
+                    seen.add(key)
+                    modname = qualified_name(module)
+                    if modname == '__main__':
+                        modname = exporter.dotpath
+                    out.append((key, value, modname, thaw_name(key)))
+    return tuple(out)
+
+@inline.fixture
+def frozenclassnames():
+    """ Same as the `frozenclasses()` fixture but with just the names """
+    out = list()
+    for quadruple in frozenclasses():
+        out.append((quadruple[0], quadruple[2], quadruple[3]))
+    return tuple(out)
+
 @inline.fixture
 def arbitrary():
     return {
@@ -323,9 +415,24 @@ def test():
     
     @inline.precheck
     def show_nestedmaps():
-        print("Nested maps fixture output:")
+        """ Nested maps fixture contents """
         pprint(nestedmaps())
     
+    @inline.precheck
+    def show_frozenclasses():
+        """ Frozen classes (by name) found in sys.modules """
+        from clu.config.env import FrozenEnviron
+        
+        freezer = tuple(frozenclasses())
+        for popsicle in freezer:
+            pprint(tuplize(popsicle[0], popsicle[1]))
+
+        print()
+        
+        fridge = tuple(frozenclassnames())
+        for meat in fridge:
+            pprint(meat)
+    
     @inline
     def test_mapwalk():
         """ Simple “mapwalk(…)” content """
@@ -336,9 +443,9 @@ def test():
     def test_mapwalk_content():
         """ Verbose “mapwalk(…)” content check """
         for mappingpath in mapwalk(nestedmaps()):
-            *namespaces, key, value = mappingpath
-            nskey = pack_ns(key, *namespaces)
-            print("NAMESPACES:", ", ".join(namespaces))
+            *fragments, key, value = mappingpath
+            nskey = ns.pack_ns(key, *fragments)
+            print("NAMESPACES:", ", ".join(fragments))
             print("KEY:", key)
             print("NSKEY:", nskey)
             print("VALUE:", value)
@@ -350,8 +457,8 @@ def test():
         flat_dict = {}
         
         for mappingpath in mapwalk(nestedmaps()):
-            *namespaces, key, value = mappingpath
-            nskey = pack_ns(key, *namespaces)
+            *fragments, key, value = mappingpath
+            nskey = ns.pack_ns(key, *fragments)
             flat_dict[nskey] = value
         
         assert flat_dict == flatdict()
@@ -359,30 +466,20 @@ def test():
         flat = FrozenFlat(flat_dict)
         nested = Nested(flat)
         
-        # print("FLAT:")
-        # pprint(flat)
-        # print()
-        
-        # print("NESTED:")
-        # pprint(nested)
-        # print()
-        
-        # print("NESTED FLATTEN:")
-        # pprint(nested.flatten())
-        # print()
-        
         assert nested.flatten() == flat
         assert nested.flatten().dictionary == flatdict()
-        assert nested == flat
+        assert nested.flatten().thaw() == flat
+        
+        assert len(flat) == len(nested)
     
     @inline
-    def test_flat_frozenflat_nested_eq():
+    def test_flat_nested_eq():
         """ Flat, FrozenFlat and Nested equivalence """
         flat_dict = {}
         
         for mappingpath in mapwalk(nestedmaps()):
-            *namespaces, key, value = mappingpath
-            nskey = pack_ns(key, *namespaces)
+            *fragments, key, value = mappingpath
+            nskey = ns.pack_ns(key, *fragments)
             flat_dict[nskey] = value
         
         assert flat_dict == flatdict()
@@ -403,8 +500,8 @@ def test():
         nested = FrozenNested(tree=nestedmaps())
         
         for mappingpath in mapwalk(nested.tree):
-            *namespaces, key, value = mappingpath
-            nskey = pack_ns(key, *namespaces)
+            *fragments, key, value = mappingpath
+            nskey = ns.pack_ns(key, *fragments)
             assert nskey in nested
     
     @inline
@@ -413,8 +510,8 @@ def test():
         nested = Nested(tree=nestedmaps(), **arbitrary())
         
         for mappingpath in mapwalk(nested.tree):
-            *namespaces, key, value = mappingpath
-            nskey = pack_ns(key, *namespaces)
+            *fragments, key, value = mappingpath
+            nskey = ns.pack_ns(key, *fragments)
             assert nskey in nested or nskey in arbitrary()
     
     @inline
@@ -423,7 +520,7 @@ def test():
         nested = FrozenNested(tree=nestedmaps())
         flat = Flat(nested)
         assert flat.nestify() == nested
-        assert flat == nested
+        assert flat.nestify().thaw() == nested
     
     @inline
     def test_nested_frozennested_eq():
@@ -438,16 +535,59 @@ def test():
     
     @inline
     def test_nested_submaps():
+        """ Check submap(…) equality """
         nested = Nested(tree=nestedmaps())
         frozen_nested = nested.freeze()
         
         assert frozen_nested.submap() == dict(frozen_nested)
         assert nested.submap('body') == frozen_nested.submap('body')
-        assert nested.submap('body', 'declare_i') == frozen_nested.submap('body', 'declare_i')
-        assert nested.submap('WTF', 'HAX') == {}
-        assert nested.submap(unprefixed=True) == { 'type' : 'Program' }
+        assert nested.submap('body:declare_i') == frozen_nested.submap('body:declare_i')
+        assert nested.submap('WTF:HAX') == {}
+        assert frozen_nested.submap('WTF:HAX') == {}
+        assert nested.submap(unprefixed=True) == FrozenNested({ 'type' : 'Program' })
         assert nested.submap() == dict(nested)
     
+    @inline.diagnostic
+    def show_nested_contents():
+        """ Evaluate submap(…) contents """
+        nested = Nested(tree=nestedmaps())
+        frozen_nested = nested.freeze()
+        
+        typ0 = typeof(nested).__name__
+        print(f"NESTED ({typ0}) SUBMAP:")
+        print(nested.submap('body:declare_i'))
+        print()
+        
+        typ1 = typeof(frozen_nested).__name__
+        print(f"FROZEN NESTED ({typ1}) SUBMAP:")
+        print(frozen_nested.submap('body:declare_i'))
+        print()
+        
+        print(f"NESTED ({typ0}) TREE:")
+        pprint(nested.tree)
+        print()
+        
+        print(f"FROZEN NESTED ({typ1}) TREE:")
+        pprint(nested.tree)
+        print()
+    
+    @inline.diagnostic
+    def show_function_cache_stats():
+        """ Show stats for the cached freeze/thaw functions """
+        from clu.naming import nameof
+        
+        # Which functions?
+        cached = (thaw_name, thaw_class,
+                  freeze_name, freeze_class)
+        total = len(cached)
+        
+        # Let’s see them:
+        for idx, function in enumerate(cached):
+            if idx > 0:
+                print()
+            print(f"FUNCTION CACHE INFO: {nameof(function)} ({idx+1} of {total})")
+            print(function.cache_info())
+    
     @inline.diagnostic
     def show_fixture_cache_stats():
         """ Show the per-fixture-function cache stats """
@@ -455,11 +595,11 @@ def test():
         for idx, name in enumerate(inline.fixtures.keys()):
             if idx > 0:
                 print()
-            print(f"FUNCTION CACHE INFO: {name} ({idx+1} of {total})")
+            print(f"FIXTURE CACHE INFO: {name} ({idx+1} of {total})")
             print(inline.fixtures[name].cache_info())
     
     # Run all inline tests:
     return inline.test(100)
 
 if __name__ == '__main__':
-    sys.exit(test())
\ No newline at end of file
+    sys.exit(test())
