@@ -81,6 +81,7 @@ starmods = ('clu.repl.ansi',
             'clu.config.env',
             'clu.config.keymap',
             'clu.config.proxy',
+            'clu.config.keymaputils',
             'clu.dicts', 'clu.exporting',
             'clu.importing', 'clu.naming', 'clu.predicates', 'clu.typology',
             'clu.fs.filesystem', 'clu.fs.misc',
@@ -108,6 +109,7 @@ mods = ('clu',
         'clu.scripts.ansicolors',
         'clu.scripts.treeline',
         'clu.stdio',
+        'clu.trie',
         'clu.typespace.types', # not technically a module!
         'clu.version', 'clu.version.git_version',
         'sys', 'os', 'io', 're', 'abc',
